<?php

declare(strict_types=1);

namespace Z14\Z14ComvahroBewerber3\Controller;

use GoetasWebservices\Xsd\XsdToPhpRuntime\Jms\Handler\BaseTypesHandler;
use GoetasWebservices\Xsd\XsdToPhpRuntime\Jms\Handler\XmlSchemaDateHandler;
use JMS\Serializer\Handler\HandlerRegistryInterface;
use JMS\Serializer\SerializerBuilder;
use Z14\Z14ComvahroBewerber3\Xml\Jobs;

/**
 * This file is part of the "comvahro-bewerber3" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * (c) 2023 
 */

/**
 * RenderController
 */
class RenderController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{

    /**
     * renderRepository
     *
     * @var \Z14\Z14ComvahroBewerber3\Domain\Repository\RenderRepository
     */
    protected $renderRepository = null;

    /**
     * @param \Z14\Z14ComvahroBewerber3\Domain\Repository\RenderRepository $renderRepository
     */
    public function injectRenderRepository(\Z14\Z14ComvahroBewerber3\Domain\Repository\RenderRepository $renderRepository)
    {
        $this->renderRepository = $renderRepository;
    }

    /**
     * @param $path
     */
    private function getDeserializedJobs($path)
    {

        // $selection2 = $this->request->getArguments(); //('selection2');
        $xmlString = file_get_contents($path);
        $serializerBuilder = SerializerBuilder::create();
        $serializerBuilder->addMetadataDir(
        __DIR__ . '/../../metadata/Xml',
        'Z14\\Z14ComvahroBewerber3\\Xml'
        );
        $serializerBuilder->configureHandlers(
        function (HandlerRegistryInterface $handler) use($serializerBuilder) {
            $serializerBuilder->addDefaultHandlers();

            // XMLSchema List handling
            $handler->registerSubscribingHandler(new BaseTypesHandler());

            // XMLSchema date handling
            $handler->registerSubscribingHandler(new XmlSchemaDateHandler());
        }
        );
        $serializer = $serializerBuilder->build();

        /* @var $jobs Jobs */
        $jobs = $serializer->deserialize($xmlString, Jobs::class, 'xml');

        return $jobs;

    } 

    private function getSelectionFilters(
        $settings,
        $selection1 = 0,
        $selection2 = 0,
        $selection3 = 0,
        $selection4 = 0,
        $selection5 = 0,
        $selection6 = 0,
        $selection7 = 0,
        $selection8 = 0,
        $selection9 = 0,
        $selection10 = 0,
        $jobs = null
    ) {

        $deserializedJobs = $this->getDeserializedJobs($_SERVER['DOCUMENT_ROOT'] . $settings['persistence']['xml']);

        $jobs = $jobs ?? $deserializedJobs->getJob();

        $filters = [];

        for ($i = 1; $i <= 10; $i++) {

            // if ((int) $settings['filter']['selection' . $i] === 0) {
            //     continue;
            // }

            $filters['selection' . $i] = [
                'selected' => ${'selection' . $i},
                'options' => [],
            ];

        }

        foreach ($jobs as $job) {

            for ($i = 1; $i <= 10; $i++) {

                if ((int) $settings['filter']['selection' . $i]['active'] === 0) {
                    continue;
                }
                
                if (array_key_exists('selection' . $i, $filters) && array_key_exists(0, $filters['selection' . $i]['options']) === false) {
                    if ((${'jobSelection' . $i . 'Description'} = $job->{'getSelection' . $i . 'Description'}()) !== null) {
                        $filters['selection' . $i]['options'][0] = ${'jobSelection' . $i . 'Description'};
                    }
                }

                if (array_key_exists('selection' . $i, $filters) && array_key_exists((${'jobSelection' . $i} = $job->{'getSelection' . $i}()), $filters['selection' . $i]) === false) {
                    if (${'jobSelection' . $i} !== null) {
                        $filters['selection' . $i]['options'][${'jobSelection' . $i}] = $job->{'getSelection' . $i . 'Translation'}();
                    }
                }

                if (array_key_exists('selection' . $i, $filters) && ${'selection' . $i} > 0) {
                    $filters['selection' . $i]['selected'] = ${'selection' . $i};
                    // if (${'jobSelection' . $i} != (int) ${'selection' . $i}) {
                    //     continue 2;
                    // }
                }

            }

        }

        return $filters;

    }

    /**
     * action list
     * 
     * @param int $page
     * @param int $selection1
     * @param int $selection2
     * @param int $selection3
     * @param int $selection4
     * @param int $selection5
     * @param int $selection6
     * @param int $selection7
     * @param int $selection8
     * @param int $selection9
     * @param int $selection10
     *
     * @return string|object|null|void
     */
    public function listAction(
        $page = 1,
        $selection1 = 0,
        $selection2 = 0,
        $selection3 = 0,
        $selection4 = 0,
        $selection5 = 0,
        $selection6 = 0,
        $selection7 = 0,
        $selection8 = 0,
        $selection9 = 0,
        $selection10 = 0
    ) {

        $settings = $this->settings;

        $deserializedJobs = $this->getDeserializedJobs($_SERVER['DOCUMENT_ROOT'] . $settings['persistence']['xml']);

        /* Z14\Z14ComvahroBewerber3\Xml\Jobs\JobsAType\JobAType[] $jobs */
        $jobs = [];

        foreach ($deserializedJobs->getJob() as $job) {

            for ($i = 1; $i <= 10; $i++) {

                ${'jobSelection' . $i} = $job->{'getSelection' . $i}();

                // Filter is set
                if (${'selection' . $i} !== 0) {
                    
                    // Job does not match the filter
                    if (${'jobSelection' . $i} !== ${'selection' . $i}) {
                        continue 2;
                    }

                }

            }

            $orderByDate = $job->getPublishFromDate() ?? $job->getLastUpdate();

            $jobs[$orderByDate->getTimestamp() . '_' . $job->getId()] = $job;

        }

        krsort($jobs);

        $this->view->assign('pageUid', $pageUid = $GLOBALS['TSFE']->id);
        $this->view->assign('jobs', $jobs);
        $this->view->assign(
            'filters', 
            $this->getSelectionFilters(
                $settings,
                $selection1,
                $selection2,
                $selection3,
                $selection4,
                $selection5,
                $selection6,
                $selection7,
                $selection8,
                $selection9,
                $selection10//,
                //$jobs
            )
        );

        $this->view->assign('data', $data = ($contentObject = $this->configurationManager->getContentObject())->data);

    }

    /**
     * action show
     *
     * @param string $uid
     * @return string|object|null|void
     */
    public function showAction(string $uid)
    {

        $settings = $this->settings;

        /* @var $job Z14\Z14ComvahroBewerber3\Xml\Jobs\JobsAType\JobAType|null */
        $job = null;

        $deserializedJobs = $this->getDeserializedJobs($_SERVER['DOCUMENT_ROOT'] . $settings['persistence']['xml']);
        
        foreach ($deserializedJobs->getJob() as $item) {

            if ($item->getId() === $uid) {
                $job = $item;
                break;
            }

        }

        $this->view->assign('job', $job);

        $this->view->assign('data', $data = ($contentObject = $this->configurationManager->getContentObject())->data);

        $this->view->assign('settings', $settings);
        
    }

    /**
     * @param $i
     * @param $settings
     */
    private function getBoxesForSelection($i, $settings)
    {

        $deserializedJobs = $this->getDeserializedJobs($_SERVER['DOCUMENT_ROOT'] . $settings['persistence']['xml']);

        $boxes = [];

        $orders = $this->getOrderForSelection($settings);

        foreach ($deserializedJobs->getJob() as $job) {

            if ((${'jobSelection' . $i} = $job->{'getSelection' . $i}()) !== null) {

                $selection = ${'jobSelection' . $i};
                $key = 999 . $selection;

                if (array_key_exists($selection, $orders)) {
                    $key = $orders[$selection];
                }

                if (array_key_exists($key, $boxes) === false) {
                    $boxes[$key] = [
                        'id' => $id = ${'jobSelection' . $i},
                        'label' => $job->{'getSelection' . $i . 'Translation'}(),
                        'outputArguments' => $this->getOutputArgumentsForBox($i, $id),
                    ];
                }

            }

        }

        ksort($boxes);

        return $boxes;

    }

    private function getOrderForSelection($settings)
    {
        $orders = [];

        if (!isset($settings['order'])) {
            return $orders;
        }

        if (empty($settings['order'])) {
            return $orders;
        }

        $list = $settings['order'];

        // Remove whitespaces before exploding the list by comma
        $items = explode(",", str_replace(' ', '', $list));

        // Use array_map to explode each item by pipe and trim resulting parts
        array_map(function($item) use (&$orders) {
            $parts = explode("|", $item);
            $orders[trim($parts[0])] = trim($parts[1]);
        }, $items);

        return $orders;

    }

    /**
     * @param $i
     * @param $settings
     */
    private function getOutputArgumentsForBox($selection, $id)
    {
        return [
            'page' => 1,
            'selection1' => $selection === 1 ?  $id : 0,
            'selection2' => $selection === 2 ?  $id : 0,
            'selection3' => $selection === 3 ?  $id : 0,
            'selection4' => $selection === 4 ?  $id : 0,
            'selection5' => $selection === 5 ?  $id : 0,
            'selection6' => $selection === 6 ?  $id : 0,
            'selection7' => $selection === 7 ?  $id : 0,
            'selection8' => $selection === 8 ?  $id : 0,
            'selection9' => $selection === 9 ?  $id : 0,
            'selection10' => $selection === 10 ?  $id : 0,
        ];
    }

    private function assignVariablesForSelectionToView(int $selection)
    {
        $request = $GLOBALS['TYPO3_REQUEST'];
        $frontendController = $request->getAttribute('frontend.controller');
        $currentPageUid = $frontendController->id;

        $this->view->assign('settings', $settings = $this->settings);

        $this->view->assign('pageUid', empty($pageUid = $this->settings['target']['pageUid']) ? $currentPageUid : $pageUid);

        $this->view->assign('pluginName', empty($pluginName = $this->settings['target']['pluginName']) ? 'jobsselection' . $selection : $pluginName);

        $this->view->assign('data', $data = ($contentObject = $this->configurationManager->getContentObject())->data);

        $this->view->assign('boxes', $this->getBoxesForSelection($selection, $settings));
    }

    /**
     * action selection1
     *
     * @return string|object|null|void
     */
    public function selection1Action()
    {

        /**
         * @var int $selection 
         * 
         * __METHOD__: This is a magic constant in PHP that returns the fully qualified name of the current class and method. 
         * It's in the form of ClassName::methodName.
         * 
         * explode("::", __METHOD__): The explode function is used to split a string into an array based on a delimiter. 
         * In this case, it's splitting the __METHOD__ *string using the :: delimiter, which effectively separates the class name 
         * from the method name. So, if __METHOD__ is "SomeClass::someMethod", after this line, $method will contain "someMethod".
         * 
         * $method = explode("::", __METHOD__)[1]: This line assigns the second element of the array returned by explode (the method name)
         * to the variable $method. After this line, $method will contain the extracted method name. 
         * 
         * filter_var($method, FILTER_SANITIZE_NUMBER_INT): The filter_var function is used to filter a variable using a specified filter. 
         * In this case, the filter FILTER_SANITIZE_NUMBER_INT is used, which removes all characters except digits, plus and minus signs. 
         * So if $method contained something like "method123", after this line, it would be sanitized to "123".
         * 
         * (int) filter_var(...): The (int) in front of the filter_var call is used to cast the result of the filtered variable to an integer. So, if the sanitized string was "123", after this line, $selection will hold the integer 123.
         * 
         * In summary, this line of code extracts the method name from the __METHOD__ constant, sanitizes it to keep only the numeric characters, and then converts the resulting sanitized string into an integer. The resulting integer is stored in the $selection variable. 
         * 
         */
        $selection = (int) filter_var(($method = explode("::", __METHOD__)[1]), FILTER_SANITIZE_NUMBER_INT);

        $this->assignVariablesForSelectionToView($selection);
    }

    /**
     * action selection2
     *
     * @return string|object|null|void
     */
    public function selection2Action()
    {
        $selection = (int) filter_var(($method = explode("::", __METHOD__)[1]), FILTER_SANITIZE_NUMBER_INT);

        $this->assignVariablesForSelectionToView($selection);
    }

    /**
     * action selection3
     *
     * @return string|object|null|void
     */
    public function selection3Action()
    {
        $selection = (int) filter_var(($method = explode("::", __METHOD__)[1]), FILTER_SANITIZE_NUMBER_INT);

        $this->assignVariablesForSelectionToView($selection);
    }

    /**
     * action selection4
     *
     * @return string|object|null|void
     */
    public function selection4Action()
    {
        $selection = (int) filter_var(($method = explode("::", __METHOD__)[1]), FILTER_SANITIZE_NUMBER_INT);

        $this->assignVariablesForSelectionToView($selection);
    }

    /**
     * action selection5
     *
     * @return string|object|null|void
     */
    public function selection5Action()
    {
        $selection = (int) filter_var(($method = explode("::", __METHOD__)[1]), FILTER_SANITIZE_NUMBER_INT);

        $this->assignVariablesForSelectionToView($selection);
    }

    /**
     * action selection6
     *
     * @return string|object|null|void
     */
    public function selection6Action()
    {
        $selection = (int) filter_var(($method = explode("::", __METHOD__)[1]), FILTER_SANITIZE_NUMBER_INT);

        $this->assignVariablesForSelectionToView($selection);
    }

    /**
     * action selection7
     *
     * @return string|object|null|void
     */
    public function selection7Action()
    {
        $selection = (int) filter_var(($method = explode("::", __METHOD__)[1]), FILTER_SANITIZE_NUMBER_INT);

        $this->assignVariablesForSelectionToView($selection);
    }

    /**
     * action selection8
     *
     * @return string|object|null|void
     */
    public function selection8Action()
    {
        $selection = (int) filter_var(($method = explode("::", __METHOD__)[1]), FILTER_SANITIZE_NUMBER_INT);

        $this->assignVariablesForSelectionToView($selection);
    }

    /**
     * action selection9
     *
     * @return string|object|null|void
     */
    public function selection9Action()
    {
        $selection = (int) filter_var(($method = explode("::", __METHOD__)[1]), FILTER_SANITIZE_NUMBER_INT);

        $this->assignVariablesForSelectionToView($selection);
    }

    /**
     * action selection10
     *
     * @return string|object|null|void
     */
    public function selection10Action()
    {
        $selection = (int) filter_var(($method = explode("::", __METHOD__)[1]), FILTER_SANITIZE_NUMBER_INT);

        $this->assignVariablesForSelectionToView($selection);
    }
}
